﻿using NLog;
using NLog.Config;
using NLog.Targets;
using System;
using System.Collections.Generic;

namespace Mapzania.Logging
{
    public class NLogLogger : Mapzania.Logging.ILogger
    {
        List<string> _customNames = new List<string>();
        string _dir;

        public NLogLogger(string dir)
        {
            if (dir == null)
                throw new ArgumentNullException("dir");

            _dir = dir;

            makeTarget("Message");
            makeTarget("Debug");
            makeTarget("Error");
        }

        public void Message(string msg, string module)
        {
            log("Message", msg, module);
        }

        public void Debug(string msg, string module)
        {
            log("Debug", msg, module);
        }

        public void Error(string msg, string module)
        {
            log("Error", msg, module);
        }

        public void Custom(string logName, string msg, string module)
        {
            if (!_customNames.Contains(logName))
                makeTarget(logName);

            log(logName, msg, module);
        }

        private void log(string logName, string msg, string module)
        {
            var logger = LogManager.GetLogger(logName);
            logger.Log(LogLevel.Info, string.Format("{0}|{1}", module, msg));
        }

        private void makeTarget(string name)
        {
            var config = LogManager.Configuration ?? new LoggingConfiguration();
            var target = new FileTarget();

            config.AddTarget(name, target);

            target.FileName = string.Format("{0}\\{1}.log", _dir, name);
            target.Layout = "${date}|${message}";

            var rule = new LoggingRule(name, LogLevel.Info, target);

            config.LoggingRules.Add(rule);

            LogManager.Configuration = config;
        }
    }
}