﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------
    ns.DrawingEngine = DrawingEngine;
    return ns;

    //---------------------------------------------------------

    function DrawingEngine(layer) {

        this.run = function (featureCollection, state) {

            var res = ns.GeoJson.empty();

            featureCollection.features.forEach(function (ftr) {

                var themer = layer.getThemer();
                var theme = themer.getTheme(state.zoom, ns.GeoJson.getId(ftr));

                if (theme) {
                    var symbol = theme.getSymbol(ftr);

                    if (symbol)
                        ns.Util.arrayAppend(_unpack(symbol, ftr), res.features);
                }
            })

            res.features.sort(function (a, b) {
                return a.level - b.level;
            });

            return res;
        }

        //---------------------------------------------------------

        function _unpack(symbol, feature) {

            var features = [];

            switch (symbol.type) {
                case "PointSymbol":
                    _unpackPoint(features, symbol, feature, 0);
                    break;
                case "CompoundPointSymbol":
                    _unpackCompoundPoint(features, symbol, feature, 0);
                    break;
                case "LineSymbol":
                    _unpackLine(features, symbol, feature, 0);
                    break;
                case "CompoundLineSymbol":
                    _unpackCompoundLine(features, symbol, feature, 0);
                    break;
                case "FillSymbol":
                    _unpackFill(features, symbol, feature, 0);
                    break;
                default:
                    ns.Util.logAndThrow("Invalid symbol type: " + symbol.type,
                        "DrawingEngine.run()");
            }

            return features;
        }

        function _unpackPoint(features, symbol, feature, level) {
            var ftr = ns.Util.assign({}, feature);
            var sym = ns.Util.assign({}, symbol);

            if (ns.Util.isNullOrUndefined(sym.rotation))
                sym.rotation = { angle: 0, originX: 0, originY: 0 };

            if (!ns.Util.isNullOrUndefined(feature.properties.rotation))
                sym.rotation.angle += 90 - ns.GeoJson.Math.radiansToDegrees(feature.properties.rotation);

            ftr.symbol = sym;
            ftr.level = level;
            features.push(ftr);
        }

        function _unpackCompoundPoint(features, symbol, feature, level) {

            var pointLevel = level;
            for (var i = symbol.points.length - 1; i >= 0; i--) {
                _unpackPoint(features, symbol.points[i], feature, pointLevel++);
            }
        }

        function _unpackLine(features, symbol, feature, level) {

            var GJ = Mapzania.GeoJson;
            var lineLength;

            // Add line
            var ftr = ns.Util.assign({}, feature);
            ftr.symbol = ns.Util.assign({}, symbol);
            ftr.level = level;
            features.push(ftr);

            if (symbol.intervalSymbol && symbol.intervalSymbol.points.length > 0 && symbol.intervalSymbolSpacing) {

                lineLength = lineLength || GJ.getLength(feature.geometry);

                var spacing = symbol.intervalSymbolSpacing;

                var length = lineLength - spacing;
                var current = length;

                while (current > 0) {
                    ftr = GJ.getFeatureAt(feature.geometry, current);
                    _unpackCompoundPoint(features, symbol.intervalSymbol, ftr, level);
                    current -= spacing;
                }
            }

            if (symbol.tailSymbol && symbol.tailSymbol.points.length > 0) {
                ftr = GJ.getFeatureAt(feature.geometry, 0);
                _unpackCompoundPoint(features, symbol.tailSymbol, ftr, level);
            }

            if (symbol.headSymbol && symbol.headSymbol.points.length > 0) {
                lineLength = lineLength || GJ.getLength(feature.geometry);
                ftr = GJ.getFeatureAt(feature.geometry, lineLength);
                _unpackCompoundPoint(features, symbol.headSymbol, ftr, level);
            }
        }

        function _unpackCompoundLine(features, symbol, feature, level) {

            var lineLevel = level;

            for (var i = symbol.lines.length - 1; i >= 0; i--) {
                _unpackLine(features, symbol.lines[i], feature, lineLevel++);
            }
        }

        function _unpackFill(features, symbol, feature, level) {

            //Inside fill
            var ftr = ns.Util.assign({}, feature);
            ftr.symbol = ns.Util.assign({}, symbol);
            ftr.symbol.outline = null;
            ftr.level = level;
            features.push(ftr);

            if (symbol.outline) {
                if (symbol.outline.type == "CompoundLineSymbol") {
                    _unpackCompoundLine(features, symbol.outline, feature, level + 1);
                }

                if (symbol.outline.type == "LineSymbol") {
                    _unpackLine(features, symbol, feature, level + 1);
                }
            }
        }
    }

})(Mapzania || {});
