﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------
    ns.Engine = Engine;
    return ns;

    //---------------------------------------------------------

    function Engine(factory) {

        var view = factory.getView();
        var mouseEngine = factory.getMouseEngine();
        var schema = factory.getSchema();
        var layerManager = factory.getLayerManager();

        var me = this;

        ns.Util.eventify(this, "Engine");

        view.fitToBoundingBox(schema.getInitialBoundingBox());

        view.on("changed", function (state) {

            _updateAll(state);

            me.fire("view_changed", {
                boundingBox: [].concat(state.bounds),
                zoom: state.zoom,
                center: Object.assign({}, state.center)
            });
        })

        layerManager.on("layer_added", function (data) {
            view.addLayer(data.layer, data.index);
        })

        layerManager.on("layer_removed", function (layer) {
            view.removeLayer(layer);
        })

        layerManager.on("layer_shown", function (layer) {
            view.showLayer(layer);
            _update(layer, view.getState());
        })

        layerManager.on("layer_hidden", function (layer) {
            view.hideLayer(layer);
        })

        layerManager.on("layer_replaced", function (data) {
            view.replaceLayer(data.key, data.layer);
        })

        layerManager.on("layer_brought_forwards", function (layer) {
            view.bringLayerForwards(layer);
        })

        layerManager.on("layer_sent_backwards", function (layer) {
            view.sendLayerBackwards(layer);
        })

        layerManager.getLayers().forEach(function (layer) {
            view.addLayer(layer);
        });

        var drawingLayer;

        //TODO: Slow this down??
        _updateAll(view.getState());

        ns.Log.debug("Object created.", "Engine");

        //-----------------------------------------------------

        this.fitToBoundingBox = function (bbox, buffer) {
            var bounds = buffer ? ns.GeoJson.buffer(bbox, buffer) : bbox;
            view.fitToBoundingBox(bounds);
        };

        this.getBoundingBox = function () {
            return view.getState().bounds;
        }

        this.getInitialBoundingBox = function () {
            return schema.getInitialBoundingBox();
        }

        this.updateLayer = function (layerKey) {
            _update(layerManager.getLayer(layerKey), view.getState());
        }

        this.refresh = function () {
            _updateAll(view.getState());
        }

        this.reset = function () {
            view.fitToBoundingBox(schema.getInitialBoundingBox());
        }

        this.zoomIn = function () {
            view.zoomIn();
        }

        this.zoomOut = function () {
            view.zoomOut();
        }

        this.setBackColor = function (color) {
            view.setBackColor(color);
        }

        this.getStateFilters = function (layerKey) {
            var layer = layerManager.getLayer(layerKey);
            var fb = factory.createFilterBuilder(layer, view.getState());
            return fb.getStateFilters();
        }

        //---------------------------------------------------------

        function _updateAll(state) {
            layerManager.getLayers().forEach(function (layer) {
                _update(layer, state);
            })
        }

        function _update(layer, state) {

            if (layer.getType() != "VectorLayer" || !layer.isVisible())
                return;

            var layerKey = layer.getKey();
            var visible = layer.isVisible();

            var fe = layer.getFeatureEngine();


            var fb = factory.createFilterBuilder(layer, state);
            var filters = fb.getRequestFilters();

            fe.run(filters)
                .then(function (data) {
                    var toDraw = new ns.DrawingEngine(layer).run(data, state);
                    view.drawLayer(layerKey, toDraw);

                    var labels = new ns.LabelEngine(layerManager, view).run();
                    view.drawLabels(labels);

                    me.fire("layer_changed", { key: layerKey, data: data });
                })
        }
    }

})(Mapzania || {});