﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    ns.LayerManager = LayerManager;
    return ns;

    //---------------------------------------------------------

    function LayerManager(api, schema, sources) {

        var me = this;

        var layers = [];

        this.getLayers = getLayers;
        this.getLayer = getLayer;
        this.showLayer = showLayer;
        this.hideLayer = hideLayer;
        this.addLayer = addLayer
        this.removeLayer = removeLayer;
        this.sendLayerBackwards = sendLayerBackwards;
        this.bringLayerForwards = bringLayerForwards;
        this.replaceLayer = replaceLayer;
        this.getVectorLayers = getVectorLayers;
        this.getRasterLayers = getRasterLayers;

        init();

        //---------------------------------------------------------

        function init() {
            ns.Util.eventify(me, "LayerManager");

            schema.getLayers().forEach(function (layer) {
                addLayer(layer.key, layer);
            })

            ns.Log.debug("Object created.", "LayerManager");
        }

        function getLayers() {
            return [].concat(layers);
        }

        function getLayer(layerKey) {
            return layers.find(function (m) {
                return m.getKey() == layerKey;
            })
        }

        function showLayer(layerKey) {
            var layer = getLayer(layerKey);
            layer.show();
            me.fire("layer_shown", layer);
        }

        function hideLayer(layerKey) {
            var layer = getLayer(layerKey);
            layer.hide();
            me.fire("layer_hidden", layer);
        }

        function addLayer(layerKey, layerJson, index) {

            var layer;

            if (layerJson) {

                var source = sources.getSource(layerJson.sourceKey);

                if (layerJson.type == "RasterLayer") {

                    layer = new ns.RasterLayer(layerJson, source);

                    if (ns.Util.isUndefined(index)) {
                        layers.push(layer);
                    }
                    else {
                        var cnt = getVectorLayers().length;
                        layers.splice(index + cnt, 1, layer);
                    }

                } else if (layerJson.type == "VectorLayer") {

                    var themer = new ns.LayerThemer();

                    schema.getThemes(layerJson.key).forEach(function (themeJson) {
                        themer.addThemeJson(themeJson);
                    });

                    var featureEngine = new ns.FeatureEngine(api, source.key);

                    layer = new ns.VectorLayer(layerJson, featureEngine, themer, source.attribution);

                    if (ns.Util.isUndefined(index)) {
                        var cnt = getVectorLayers().length;
                        layers.splice(cnt, 0, layer);
                    }
                    else {
                        layers.splice(index, 0, layer);
                    }

                }
            }
            else {

                var defaults = {
                    key: layerKey,
                    sourceKey: layerKey + "_SRC",
                    filter: null,
                    fields: [],
                    visible: true
                };

                var json = Object.assign(defaults, layerJson);

                var themer = new ns.LayerThemer();
                var featureEngine = new ns.FeatureEngine(api, json.sourceKey);
                var attribution = sources.hasSource(json.sourceKey) ? sources.getSource(json.sourceKey).attribution : "";

                layer = new ns.VectorLayer(json, featureEngine, themer, attribution);

                if (ns.Util.isUndefined(index)) {
                    var cnt = getVectorLayers().length;
                    layers.splice(cnt, 0, layer);
                }
                else {
                    layers.splice(index, 0, layer);
                }

            }

            console.log("Layers.....", layers);

            me.fire("layer_added", { layer: layer, index: index });
        }

        function removeLayer(layerKey) {
            var layer = getLayer(layerKey);
            var idx = layers.indexOf(layer);
            layers.splice(idx, 1);
            me.fire("layer_removed", layer);
        }

        function sendLayerBackwards(layerKey) {
            var layer = getLayer(layerKey);

            var idx = layers.indexOf(layer);
            var tmp = layers[idx + 1];

            layers[idx + 1] = layer;
            layers[idx] = tmp;

            me.fire("layer_sent_backwards", layer);
        }

        function bringLayerForwards(layerKey) {
            var layer = getLayer(layerKey);

            var idx = layers.indexOf(layer);
            var tmp = layers[idx - 1];

            layers[idx - 1] = layer;
            layers[idx] = tmp;

            me.fire("layer_brought_forwards", layer);
        }

        function replaceLayer(layerKey, layerJson) {

            var layer;

            if (layerJson.type == "RasterLayer") {

                var source = sources.getSource(layerJson.sourceKey);
                layer = new ns.RasterLayer(layerJson, source);
            }

            if (layerJson.type == "VectorLayer") {

                var themer = new ns.LayerThemer();
                var featureEngine = new ns.FeatureEngine(api, layerJson.sourceKey);
                var sourceJson = sources.getSource(layerJson.sourceKey);
                layer = new ns.VectorLayer(layerJson, featureEngine, themer, sourceJson.attribution);
            }

            var existing = layers.find(m => m.getKey() == layerKey);
            var idx = layers.indexOf(existing);
            layers[idx] = layer;

            me.fire("layer_replaced", { key: layerKey, layer: layer });
        }

        function getVectorLayers() {
            return layers.filter(m => m.getType() == "VectorLayer");
        }

        function getRasterLayers() {
            return layers.filter(m => m.getType() == "RasterLayer");
        }
    }

})(Mapzania || {});
