﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    ns.MouseModes = {
        Pan: 1,
        Query: 2,
        DrawPoint: 3,
        DrawLine: 4,
        DrawPolygon: 5
    };

    ns.MouseEngine = MouseEngine;
    return ns;

    //---------------------------------------------------------

    function MouseEngine(view) {

        var GJ = ns.GeoJson;

        var me = this;

        ns.Util.eventify(this, "MouseEngine");

        var mode = ns.MouseModes.Pan;

        var drawn;

        view.on("clicked", function (e) {
            switch (mode) {

                case ns.MouseModes.DrawPoint:
                    me.fire("drawing_done", e.position);
                    drawn = e.position;
                    view.drawSelector(drawn);
                    break;

                case ns.MouseModes.DrawLine:
                    drawn = drawn || GJ.createLineString([]);
                    GJ.appendPointToLineString(drawn, e.position);
                    view.drawSelector(drawn);
                    break;

                case ns.MouseModes.DrawPolygon:
                    drawn = drawn || GJ.createPolygon([]);
                    GJ.appendPointToPolygon(drawn, e.position);
                    view.drawSelector(drawn);
                    break;
            }
        })

        view.on("double_clicked", function (e) {

            switch (mode) {
                case ns.MouseModes.DrawPoint:
                    break;

                case ns.MouseModes.DrawLine:
                    me.fire("drawing_done", drawn);
                    drawn = null;
                    break;

                case ns.MouseModes.DrawPolygon:
                    me.fire("drawing_done", drawn);
                    drawn = null;
                    break;
            }
        });

        ns.Log.debug("Object created.", "MouseEngine");

        //-----------------------------------------------------

        this.setMode = function (val) {
            mode = val;
            drawn = null;
            view.drawSelector(null);
            view.setMouseMode(val);
            this.fire("mouse_mode_set", mode);
        }

        this.getMode = function () {
            return mode;
        }
    }

})(Mapzania || {});