﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    ns.FilterBuilder = FilterBuilder;
    return ns;

    //---------------------------------------------------------

    function FilterBuilder(layer, state, options) {

        this.getRequestFilters = function () {
            return _buildFilterStack(layer.getFilters());
        }

        this.getStateFilters = function () {
            return _buildFilterStack();
        }

        this.getResponseFilters = function (layer) {

            var filters = [];

            //_addLocalFeatures
            if (layer.clientFeatures
                && layer.clientFeatures.features
                && layer.clientFeatures.features.length > 0
                && layer.requestFilters.length === 0) {

                filters.push(function (features) {
                    var res = Mapzania.GeoJson.fromFeatureCollection(layer.clientFeatures);
                    Mapzania.Projection.projectFeatureArray(res, "LATLONG", "GOOGLE");

                    //TODO: Replace based on ids
                    [].push.apply(features, res);
                });
            }

            //removeHidden
            filters.push(function (features) {

                for (var i = features.length - 1; i >= 0; i--) {
                    if (layer.hidden.indexOf(features[i].id) > -1)
                        features.splice(i, 1);
                }
            });

            //filters.push(function (features) {

            //    for (var i = features.length - 1; i >= 0; i--) {
            //        if (layer.hidden.indexOf(features[i].id) > -1)
            //            features.splice(i, 1);
            //    }
            //});

            //add custom
            for (var i = 0; i < layer.responseFilters.length; i++) {
                filters.push(layer.requestFilters[i]);
            }

            return filters;
        }

        //---------------------------------------------------------

        function _buildFilterStack(userFilters) {
            var filters = [];
            var requiresAppending = true;

            _addLocalFeatures(filters);

            if (filters.length > 0)
                requiresAppending = false;

            _addFilterByText(filters);
            _addFilterByBoundingBox(filters);
            _addChangeProperties(layer, filters);
            _addLabelVisibilityClause(filters);

            if (userFilters)
                filters = filters.concat(userFilters);

            //TODO: ChangePrecision

            if (!layer.tiled)
                _addSimplifyGeometry(filters);

            return {
                filters: filters,
                requiresAppending: requiresAppending
            }
        }

        function _addLocalFeatures(filters) {

            var featureCollection = layer.getFeatureEngine().getLocalFeatureCollection();

            if (featureCollection.features.length > 0 && layer.getFilters().length !== 0)
                filters.push({ type: "AddFeatureCollection", features: featureCollection })
        }

        function _addFilterByText(filters) {

            var layerText = layer.getFilterText();

            var themer = layer.getThemer();
            var themeText = themer.getFilterText(state.zoom);

            if (layerText && themeText)
                text = "(" + themeText + ") AND (" + layerText + ")";
            else
                if (themeText)
                    text = themeText;
                else
                    text = layerText;

            if (text) {
                var filter = ns.Filters.filterByText(text, false);
                filters.push(filter);
            }
        }

        function _addFilterByBoundingBox(filters) {

            var filter = ns.Filters.filterByBoundingBox(state.bounds);
            filters.push(filter);
        }

        function _addChangeProperties(layer, filters) {

            var fields = [];

            var layerFields = layer.getFields();

            if (layerFields)
                layerFields.forEach(function (cur) {
                    if (!cur.expression)
                        fields.push(cur.key);
                });

            var fieldsStr = null;

            if (fields.length > 0)
                fieldsStr = fields.join(",");

            if (fieldsStr)
                filters.push(ns.Filters.changeProperties(fieldsStr));
        }

        function _addLabelVisibilityClause(filters) {
            //TODO:labelVisibilityClause
        }

        function _addSimplifyGeometry(filters) {

            var bb = state.bounds;
            var pb = state.pixelBounds;

            var pW = Math.abs(pb[3] - pb[1])
            var pH = Math.abs(pb[2] - pb[0])
            var bW = Math.abs(bb[3] - bb[1])
            var bH = Math.abs(bb[2] - bb[0])

            var thinX = bW / pW;
            var thinY = bH / pH;

            var thinFactor = options.thinToNearestXPixels * Math.min(thinX, thinY);

            var filter = ns.Filters.simplifyGeometry(thinFactor);

            filters.push(filter);
        }
    }

})(Mapzania || {})