﻿/// <reference path="../../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    var GJ = ns.GeoJson = ns.GeoJson || {};

    GJ.appendPointToLineString = appendPointToLineString;
    GJ.appendPointToPolygon = appendPointToPolygon;
    GJ.empty = empty;
    GJ.createFeature = createFeature;
    GJ.createFeatureCollection = createFeatureCollection;
    GJ.createLineString = createLineString;
    GJ.createMultiLineString = createMultiLineString;
    GJ.createMultiPoint = createMultiPoint;
    GJ.createMultiPolygon = createMultiPolygon;
    GJ.createPoint = createPoint;
    GJ.createPolygon = createPolygon;
    GJ.getId = getId;
    GJ.getType = getType;
    GJ.isBoundingBox = isBoundingBox;
    GJ.merge = merge;

    GJ.Exceptions = {
        doesNotApply: doesNotApply,
        notImplemented: notImplemented,
    };

    return ns;

    //---------------------------------------------------------

    function appendPointToLineString(line, point) {
        line.coordinates.push(point.coordinates);
    }

    function appendPointToPolygon(poly, point) {

        if (poly.coordinates.length == 0)
            poly.coordinates.push([]);

        var shell = poly.coordinates[0];

        //if (shell.length == 0) {
            shell.push(point.coordinates);
        //}


        //else if (shell.length == 1) {
        //    shell.push(point.coordinates);
        //    shell.push(shell[0]);
        //}
        //else {
        //    var first = shell[0];
        //    var last = shell[shell.length - 1];

        //    if (first[0] != last[0] || first[1] != last[1])
        //        shell.push([].concat(first));

        //    shell.splice(shell.length - 2, 0, point.coordinates)
        //}
    }

    function empty() {
        return {
            type: "FeatureCollection",
            features: []
        };
    }

    function createFeature(geometry, properties, id) {
        var result =  {
            type: "Feature",
            geometry: geometry,
            properties: properties || {}
        };

        if (id)
            result.id = id;

        return result;
    }

    //source is either array of features or single feature;
    function createFeatureCollection(source) {
        return merge(source, empty());
    }

    function createLineString(coordinates) {
        return {
            type: "LineString",
            coordinates: [].concat(coordinates)
        }
    }

    function createMultiLineString(coordinates) {
        return {
            type: "MultiLineString",
            coordinates: [].concat(coordinates)
        }
    }

    function createMultiPoint(coordinates) {
        return {
            type: "MultiPoint",
            coordinates: [].concat(coordinates)
        }
    }

    function createMultiPolygon(coordinates) {
        return {
            type: "MultiPolygon",
            coordinates: [].concat(coordinates)
        }
    }

    function createPoint(param1, param2) {
        var coords;

        if (ns.Util.isArray(param1))
            coords = [].concat(param1);
        else
            coords = [param1, param2]

        return {
            type: "Point",
            coordinates: coords
        }
    }

    function createPolygon(coordinates) {

        if (!isBoundingBox(coordinates)) {

            return {
                type: "Polygon",
                coordinates: [].concat(coordinates)
            };
        }

        var minX = coordinates[0],
            minY = coordinates[1],
            maxX = coordinates[2],
            maxY = coordinates[3];

        var bottomLeft = [minX, minY];
        var topLeft = [minX, maxY];
        var topRight = [maxX, maxY];
        var bottomRight = [maxX, minY];

        var shell = [bottomLeft, topLeft, topRight, bottomRight, bottomLeft];

        return {
            type: "Polygon",
            coordinates: [shell]
        }
    }

    function getId(feature) {
        return feature.id || feature.properties["id"] || feature.properties["Id"];
    }

    function getType(geometry) {
        if (isBoundingBox(geometry))
            return "BoundingBox";

        return geometry.type;
    }

    function isBoundingBox(bbox) {
        return bbox.length == 4 &&
            ns.Util.isNumeric(bbox[0]) &&
            ns.Util.isNumeric(bbox[1]) &&
            ns.Util.isNumeric(bbox[2]) &&
            ns.Util.isNumeric(bbox[3]);
    }

    function merge(source, destination) {
        //Feature List
        if (ns.Util.isArray(source))
            destination.features.push(...source);
            //Feature
        else if (source.type == "Feature")
            destination.features.push(source);
            //FeatureCollection
        else
            ns.Util.arrayAppend(source.features, destination.features);

        return destination;
    }

    //----------------------EXCEPTIONS--------------------------

    function notImplemented(type, functionName) {
        ns.Util.logAndThrow(
            "Function (" + functionName + ") not yet implemented for type=" + type + ".",
            "GeoJson." + functionName + "()"
            );
    }

    function doesNotApply(type, functionName) {
        ns.Util.logAndThrow(
            "Function (" + functionName + ") does not apply to geometry type=" + type + ".",
            "GeoJson." + functionName + "()"
            );
    }

})(Mapzania || {});