﻿/// <reference path="../../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    var GJ = ns.GeoJson = ns.GeoJson || {};

    GJ.crop = crop;

    crop.implemented = {
        LineString: doLineString,
        MultiLineString: doMultiLineString
    };

    return ns;

    //---------------------------------------------------------

    function crop(geometry, boundary) {

        var type = GJ.getType(geometry);
        var func = crop.implemented[type];

        if (!func)
            GJ.Exceptions.notImplemented(type, "crop");

        return func(geometry, boundary);
    }

    //---------------------------------------------------------

    function doLineString(geometry, boundary) {

        var coords = _cropCoords(geometry.coordinates, boundary);

        if (coords.length == 0)
            return null;

        return {
            type: "LineString",
            coordinates: coords
        };
    }

    function doMultiLineString(geometry, boundary) {
        var lines = [];

        geometry.coordinates.forEach(function (line) {

            var coords = _cropCoords(line, boundary);

            if (coords.length != 0)
                lines.push(coords);
        })

        if (lines.length == 0)
            return null;

        return {
            type: "MultiLineString",
            coordinates: lines
        };
    }

    //---------------------------------------------------------

    function _cropCoords(coords, bbox) {

        var result = [];

        if (coords.length < 2)
            return [];

        for (var i = 1; i < coords.length; i++) {

            var start = coords[i - 1];
            var end = coords[i];

            var startIsContained = GJ.isContainedBy(GJ.createPoint(start), bbox);
            var endIsContained = GJ.isContainedBy(GJ.createPoint(end), bbox);

            //Both points inside
            if (startIsContained && endIsContained) {
                result.push(start);
                result.push(end);
                continue;
            }

            // Both points outside
            if (!startIsContained && !endIsContained) {
                continue;
            }

            var minX = bbox[0], minY = bbox[1], maxX = bbox[2], maxY = bbox[3];

            var bottomLeft = [minX, minY];
            var topLeft = [minX, maxY];
            var topRight = [maxX, maxY];
            var bottomRight = [maxX, minY];

            var xpoint = GJ.Math.segmentIntersection(start, end, bottomLeft, topLeft) ||
                GJ.Math.segmentIntersection(start, end, topLeft, topRight) ||
                GJ.Math.segmentIntersection(start, end, topRight, bottomRight) ||
                GJ.Math.segmentIntersection(start, end, bottomRight, bottomLeft);

            if (startIsContained) {
                result.push(start);
                result.push(xpoint);
            }
            else {
                result.push(xpoint);
                result.push(end);
            }
        }

        return result;
    }


})(Mapzania || {});