﻿/// <reference path="../../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    var GJ = ns.GeoJson = ns.GeoJson || {};
    GJ.getArea = getArea;

    getArea.implemented = {
        Point: zero,
        MultiPoint: zero,
        LineString: zero,
        MultiLineString: zero,
        Polygon: doPolygon,
        MultiPolygon: doMultiPolygon,
        GeometryCollection: doGeometryCollection
    };

    return ns;

    //---------------------------------------------------------

    function getArea(geometry) {

        var type = GJ.getType(geometry);
        var func = getArea.implemented[type];

        if (!func)
            GJ.Exceptions.notImplemented(type, "getArea");

        return func(geometry);
    }

    //---------------------------------------------------------

    function zero(geometry) {
        return 0;
    }

    function doPolygon(geometry) {
        //https://github.com/Turfjs/turf/tree/master/packages/turf-area

        var coords = geometry.coordinates;

        var area = 0;

        if (coords && coords.length > 0) {

            area += Math.abs(ringArea(coords[0]));

            for (var i = 1; i < coords.length; i++) {

                area -= Math.abs(ringArea(coords[i]));
            }
        }

        return area;

        function ringArea(coords) {

            var RADIUS = 6378137;

            var p1;
            var p2;
            var p3;
            var lowerIndex;
            var middleIndex;
            var upperIndex;
            var i;
            var area = 0;
            var coordsLength = coords.length;

            function rad(_) {
                return _ * Math.PI / 180;
            }

            if (coordsLength > 2) {
                for (i = 0; i < coordsLength; i++) {
                    if (i === coordsLength - 2) { // i = N-2
                        lowerIndex = coordsLength - 2;
                        middleIndex = coordsLength - 1;
                        upperIndex = 0;
                    } else if (i === coordsLength - 1) { // i = N-1
                        lowerIndex = coordsLength - 1;
                        middleIndex = 0;
                        upperIndex = 1;
                    } else { // i = 0 to N-3
                        lowerIndex = i;
                        middleIndex = i + 1;
                        upperIndex = i + 2;
                    }
                    p1 = coords[lowerIndex];
                    p2 = coords[middleIndex];
                    p3 = coords[upperIndex];
                    area += (rad(p3[0]) - rad(p1[0])) * Math.sin(rad(p2[1]));
                }

                area = area * RADIUS * RADIUS / 2;
            }

            return area;
        }
    }

    function doMultiPolygon(geometry) {

        var area = 0;

        for (i = 0; i < geometry.coordinates.length; i++) {
            area += doPolygon({ coordinates: geometry.coordinates[i] });
        }
        return area;
    }

    function doGeometryCollection(geometry) {

        var area = 0;

        for (i = 0; i < geometry.geometries.length; i++) {
            area += getArea(geometry.geometries[i]);
        }
        return area;
    }

})(Mapzania || {});