﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    var GJ = ns.GeoJson = ns.GeoJson || {};
    GJ.getBoundingBox = getBoundingBox;

    getBoundingBox.implemented = {
            BoundingBox:doBoundingBox,
            Point: doPoint,
            MultiPoint: doMultiPoint,
            LineString: doLineString,
            MultiLineString: doMultiLineString,
            Polygon: doPolygon,
            MultiPolygon: doMultiPolygon,
            GeometryCollection: doGeometryCollection
    };

    return ns;

    //---------------------------------------------------------

    function getBoundingBox(geometry) {

        var type = GJ.getType(geometry);
        var func = getBoundingBox.implemented[type];

        if (!func)
            GJ.Exceptions.notImplemented(type, "getBoundingBox");

        return func(geometry);
    }    

    //---------------------------------------------------------

    function doBoundingBox(geometry) {
        return [].concat(geometry);
    }

    function doPoint(geometry) {
        var coords = geometry.coordinates;
        return [coords[0], coords[1], coords[0], coords[1]];
    }

    function doMultiPoint(geometry) {

        var res = _empty();

        geometry.coordinates.forEach(function (coord) {
            _include(res, coord);
        })

        return res;
    }

    function doLineString(geometry) {
        return doMultiPoint(geometry);
    }

    function doMultiLineString(geometry) {

        var res = _empty();

        geometry.coordinates.forEach(function (coord) {
            var bbox = doLineString({ coordinates:coord });
            _merge(res, bbox);
        })

        return res;
    }

    function doPolygon(geometry) {

        return doMultiPoint({
            coordinates: geometry.coordinates[0]
        });
    }

    function doMultiPolygon(geometry) {

        var res = _empty();

        geometry.coordinates.forEach(function (coord) {
            var bbox = doPolygon({ coordinates: coord });
            _merge(res, bbox);
        })

        return res;
    }

    function doGeometryCollection(geometry) {

        var res = _empty();

        geometry.geometries.forEach(function (geom) {
            var bbox = bbox(geom);
            _merge(res, bbox);
        })

        return res;
    }

    //---------------------------------------------------------

    function _merge(bbox, new_bbox) {

        if (bbox[0] < new_bbox[0]) bbox[0] = new_bbox[0];
        if (bbox[1] < new_bbox[1]) bbox[1] = new_bbox[1];
        if (bbox[2] > new_bbox[2]) bbox[2] = new_bbox[2];
        if (bbox[3] > new_bbox[3]) bbox[3] = new_bbox[3];
    }

    function _include(bbox, coord) {

        if (bbox[0] > coord[0]) bbox[0] = coord[0];
        if (bbox[1] > coord[1]) bbox[1] = coord[1];
        if (bbox[2] < coord[0]) bbox[2] = coord[0];
        if (bbox[3] < coord[1]) bbox[3] = coord[1];
    }

    function _empty() {
        return [Number.MAX_VALUE, Number.MAX_VALUE, -Number.MAX_VALUE, -Number.MAX_VALUE];
    }

})(Mapzania || {});