﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    var GJ = ns.GeoJson = ns.GeoJson || {};
    GJ.getFeatureAt = getFeatureAt;

    getFeatureAt.implemented = {
        LineString: doLineString,
        MultiLineString: doMultiLineString
    };

    return ns;

    //---------------------------------------------------------

    function getFeatureAt(geometry, distanceAlong) {

        var type = GJ.getType(geometry);
        var func = getFeatureAt.implemented[type];

        if (!func)
            GJ.Exceptions.notImplemented(type, "getFeatureAt");

        return func(geometry, distanceAlong);
    }

    //---------------------------------------------------------


    function doLineString(geometry, distanceAlong) {
        var totalLength = 0.0;
        var coords = geometry.coordinates;

        for (var i = 0; i < coords.length - 1; i++) {

            var p1x = coords[i][0];
            var p1y = coords[i][1];
            var p2x = coords[i + 1][0];
            var p2y = coords[i + 1][1];

            var distance = GJ.Math.distance(p1x, p1y, p2x, p2y);
            totalLength += distance;

            if (totalLength >= distanceAlong) {

                var delta = totalLength - distanceAlong;
                var f = 1 - (delta / distance);

                var point =  GJ.createPoint([
                    p1x + (p2x - p1x) * f,
                    p1y + (p2y - p1y) * f
                ]);

                var feature = GJ.createFeature(point);

                feature.properties.rotation = Math.atan2((p2y - p1y), (p2x - p1x));

                return feature;
            }
        }

        return null;
    }

    function doMultiLineString(geometry, distanceAlong) {
        var longest = GJ.getLongest(geometry);
        return doLineString(longest, distanceAlong);
    }

})(Mapzania || {});