﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    var GJ = ns.GeoJson = ns.GeoJson || {};
    GJ.getLongest = getLongest;

    getLongest.implemented = {
        BoundingBox: doesNotApply,
        Point: doesNotApply,
        MultiPoint: doesNotApply,
        LineString: doesNotApply,
        MultiLineString: doMultiLineString,
        Polygon: doesNotApply,
        MultiPolygon: doesNotApply,
        GeometryCollection: doesNotApply
    };

    return ns;    

    //---------------------------------------------------------

    function getLongest(geometry) {

        var type = GJ.getType(geometry);
        var func = getLongest.implemented[type];

        if (!func)
            GJ.Exceptions.notImplemented(type, "getLongest");

        return func(geometry);
    }

    //---------------------------------------------------------

    function doesNotApply(geometry) {
        var type = GJ.getType(geometry);
        GJ.Exceptions.doesNotApply(type, "getLongest");
    }

    function doMultiLineString(geometry) {
        var longest = { item: null, length: 0 };

        geometry.coordinates.forEach(function (lineCoords) {

            var line = GJ.createLineString(lineCoords);
            var length = GJ.getLength(line);

            if (length > longest.length) {
                longest.item = line;
                longest.length = length;
            }
        });

        return longest.item;
    }

})(Mapzania || {});