﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    var GJ = ns.GeoJson = ns.GeoJson || {};
    GJ.intersection = intersection;

    intersection.implemented = {
        LineString: {
            LineString: doLineStringWithLineString
        }
    }

    return ns;

    //---------------------------------------------------------

    function intersection(geometry1, geometry2) {

        var type = GJ.getType(geometry1);

        var funcs = intersection.implemented[type];

        if (!funcs)
            GJ.Exceptions.notImplemented(type, "intersection");

        type = GJ.getType(geometry2);

        var func = funcs[type];

        if (!func)
            GJ.Exceptions.notImplemented(type, "intersection");

        return func(geometry1, geometry2);
    }    

    //---------------------------------------------------------

    function doLineStringWithLineString(line1, line2) {

        var coords1 = line1.coordinates;
        var coords2 = line2.coordinates;

        var points = [];

        for (var i = 1; i < coords1.length; i++) {
            var start1 = coords1[i - 1];
            var end1 = coords1[i];

            for (var j = 1; j < coords2.length; i++) {
                var start2 = coords2[j - 1];
                var end2 = coords2[j];

                var point = GJ.Math.segmentIntersection(start1, end1, start2, end2);
                if (point)
                    points.push(point);
            }
        }

        if (points.length == 0)
            return null;

        return {
            type: "MultiPoint",
            coordinates: points
        }
    }

})(Mapzania || {});