﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    var GJ = ns.GeoJson = ns.GeoJson || {};
    GJ.intersects = intersects;

    intersects.implemented = {
            BoundingBox: {
                BoundingBox: doBoundingBoxWithBoundingBox
            },
            LineString: {
                LineString: doLineStringWithLineString
            },
            Polygon: {
                Polygon: doPolygonWithPolygon
            }
        };

    return ns;

    //---------------------------------------------------------

    function intersects(geometry1, geometry2) {

        var type = GJ.getType(geometry1);

        var funcs = intersects.implemented[type];

        if (!funcs)
            GJ.Exceptions.notImplemented(type, "intersects");

        type = GJ.getType(geometry2);

        var func = funcs[type];

        if (!func)
            GJ.Exceptions.notImplemented(type, "intersects");

        return func(geometry1, geometry2);
    }

    //---------------------------------------------------------

    function doBoundingBoxWithBoundingBox(bbox1, bbox2) {

        var minX1 = bbox1[0], minY1 = bbox1[1], maxX1 = bbox1[2], maxY1 = bbox1[3];
        var minX2 = bbox2[0], minY2 = bbox2[1], maxX2 = bbox2[2], maxY2 = bbox2[3];

        return (Math.max(minY1, maxY1) > minY2) &&
            (Math.min(minY1, maxY1) < maxY2) &&
            (Math.max(minX1, maxX1) > minX2) &&
            (Math.min(minX1, maxX1) < maxX2);
    }

    function doLineStringWithLineString(line1, line2) {

        var coords1 = line1.coordinates;
        var coords2 = line2.coordinates;

        var points = [];

        for (var i = 1; i < coords1.length; i++) {
            var start1 = coords1[i - 1];
            var end1 = coords1[i];

            for (var j = 1; j < coords2.length; j++) {
                var start2 = coords2[j - 1];
                var end2 = coords2[j];

                var point = GJ.Math.segmentIntersection(start1, end1, start2, end2);
                if (point)
                    points.push(point);
            }
        }

        if (points.length == 0)
            return null;

        return {
            type: "MultiPoint",
            coordinates: points
        }

    }

    function doPolygonWithPolygon(poly1, poly2) {

        var bbox1 = GJ.getBoundingBox(poly1);
        var bbox2 = GJ.getBoundingBox(poly2);

        if (doBoundingBoxWithBoundingBox(bbox1, bbox2) == false)
            return false;

        var shell1 = GJ.createLineString(poly1.coordinates[0]);
        var shell2 = GJ.createLineString(poly2.coordinates[0]);

        if (doLineStringWithLineString(shell1, shell2))
            return true;

        if (GJ.isContainedBy(poly1, poly2))
            return true;

        if (GJ.isContainedBy(poly2, poly1))
            return true;

        return false;
    }

})(Mapzania || {});