﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    var GJ = ns.GeoJson = ns.GeoJson || {};
    GJ.isContainedBy = isContainedBy;

    isContainedBy.implemented = {
        Point: {
            BoundingBox: doPointByBoundingBox,
            LineString: doPointByRing,
            Polygon: doPointByPolygon
        },
        LineString: {
            LineString: doLineStringByRing,
        },
        Polygon: {
            Polygon: doPolygonByPolygon
        }
    };

    return ns;

    //---------------------------------------------------------

    function isContainedBy(geometry, container) {

        var type = GJ.getType(geometry);

        var funcs = isContainedBy.implemented[type];

        if (!funcs)
            GJ.Exceptions.notImplemented(type, "isContainedBy");

        type = GJ.getType(container);

        var func = funcs[type];

        if (!func)
            GJ.Exceptions.notImplemented(type, "isContainedBy");

        return func(geometry, container);
    }

    //---------------------------------------------------------

    function doPointByBoundingBox(geometry, container) {

        var bbox = container;

        var minX = bbox[0];
        var minY = bbox[1];
        var maxX = bbox[2];
        var maxY = bbox[3];

        var px = geometry.coordinates[0];
        var py = geometry.coordinates[1];

        return (px >= minX && px <= maxX) && (py >= minY && py <= maxY);
    }

    function doPointByRing(point, ring) {

        var xInt;		// x intersection of e with ray
        var crossings = 0;	// number of edge/ray crossings
        var x1, y1, x2, y2;
        var nPts = ring.length;
        var pointx = point.coordinates[0];
        var pointy = point.coordinates[1];

        // For each line edge l = (i-1, i), see if it crosses ray from test point in positive x direction.
        for (i = 1; i < nPts; i++) {
            var p1 = ring.coordinates[i];
            var p2 = ring.coordinates[i - 1];
            x1 = p1[0] - pointx;
            y1 = p1[1] - pointy;
            x2 = p2[0] - pointx;
            y2 = p2[1] - pointy;

            if (((y1 > 0) && (y2 <= 0)) ||
                ((y2 > 0) && (y1 <= 0))) {
                // e straddles x axis, so compute intersection.
                xInt = (x1 * y2 - x2 * y1) / (y2 - y1);
                //xsave = xInt;
                // crosses ray if strictly positive intersection.
                if (0.0 < xInt) {
                    crossings++;
                }
            }
        }
        // point is inside if an odd number of crossings.
        if ((crossings % 2) == 1) {
            return true;
        }
        else {
            return false;
        }
    }

    function doPointByPolygon(point, poly) {
        var ring = { coordinates: poly2.coordinates[0] };
        return doPointByRing(point, ring)
    }

    function doLineStringByRing(line, ring) {
        
        line.coordinates.forEach(function (coord) {
            if (!doPointByRing({ coordinates: coord }, ring))
                return false;
        })

        return true;
    }

    function doPolygonByPolygon(poly1, poly2) {
        var line = { coordinates: poly1.coordinates[0] };
        var ring = { coordinates: poly2.coordinates[0] };

        return doLineStringByRing(line, ring);
    }

    //---------------------------------------------------------


})(Mapzania || {});