﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    var GJ = ns.GeoJson = ns.GeoJson || {};

    GJ.Math = {};

    GJ.Math.distance = distance;
    GJ.Math.nearestPointOnSegment = nearestPointOnSegment;
    GJ.Math.segmentIntersection = segmentIntersection;

    GJ.Math.segmentize = segmentize;
    GJ.Math.desegmentize = desegmentize;

    GJ.Math.degreesToRadians = degreesToRadians;
    GJ.Math.radiansToDegrees = radiansToDegrees;

    return ns;

    //---------------------------------------------------------

    function distance(param1, param2, param3, param4) {

        if (param3) {
            x1 = param1;
            y1 = param2;
            x2 = param3;
            y2 = param4;
        }
        else {
            x1 = param1[0];
            y1 = param1[1];
            x2 = param2[0];
            y2 = param2[1];
        }

        var R = 6371000;
        deltaX = (x2 - x1) * Math.PI / 180;
        y1 = y1 * Math.PI / 180;
        y2 = y2 * Math.PI / 180;
        var x = deltaX * Math.cos((y1 + y2) / 2);
        var y = (y2 - y1);
        var d = Math.sqrt(x * x + y * y);
        return R * d;
    };

    function segmentIntersection(start1, end1, start2, end2) {
        // http://jsfiddle.net/justin_c_rounds/Gd2S2/light/

        var p1x = start1[0];
        var p1y = start1[1];
        var p2x = end1[0];
        var p2y = end1[1];
        var p3x = start2[0];
        var p3y = start2[1];
        var p4x = end2[0];
        var p4y = end2[1];


        var denominator, a, b, numerator1, numerator2, result = {
            x: null,
            y: null,
            onLine1: false,
            onLine2: false
        };

        denominator = ((p4y - p3y) * (p2x - p1x)) - ((p4x - p3x) * (p2y - p1y));
        if (denominator == 0) {
            return null;
        }
        a = p1y - p3y;
        b = p1x - p3x;
        numerator1 = ((p4x - p3x) * a) - ((p4y - p3y) * b);
        numerator2 = ((p2x - p1x) * a) - ((p2y - p1y) * b);
        a = numerator1 / denominator;
        b = numerator2 / denominator;

        // if we cast these lines infinitely in both directions, they intersect here:
        result.x = p1x + (a * (p2x - p1x));
        result.y = p1y + (a * (p2y - p1y));
        /*
            // it is worth noting that this should be the same as:
            x = p3x + (b * (p4x - p3x));
            y = p3x + (b * (p4y - p3y));
        */
        // if line1 is a segment and line2 is infinite, they intersect if:
        if (a > 0 && a < 1) {
            result.onLine1 = true;
        }
        // if line2 is a segment and line1 is infinite, they intersect if:
        if (b > 0 && b < 1) {
            result.onLine2 = true;
        }

        if (result.onLine1 && result.onLine2)
            return [result.x, result.y];
        else
            return null;
    }

    function nearestPointOnSegment(point, start, end) {

        var startX = start[0];
        var startY = start[1];

        var endX = end[0];
        var endY = end[1];

        var pointX = point[0];
        var pointY = point[1];

        lineDeltaX = endX - startX;
        lineDeltaY = endY - startY;

        var len = lineDeltaX * lineDeltaX + lineDeltaY * lineDeltaY;

        var pointDeltaX = pointX - startX;
        var pointDeltaY = pointY - startY;

        var dot = pointDeltaX * lineDeltaX + pointDeltaY * lineDeltaY;

        var t = Math.min(1, Math.max(0, dot / len));

        var resultX = startX + lineDeltaX * t;
        var resultY = startY + lineDeltaY * t;

        return [resultX, resultY];
    }

    function segmentize(lineString) {

        var result = [];

        var coords = lineString.coordinates;

        for (var i = 1; i < coords.length; i++) {
            var start = coords[i - 1];
            var end = coords[i];

            result.push([start, end]);
        }

        return result;
    }

    function desegmentize(segments) {

        var coords = [];

        for (var i = 0; i < segments.length; i++) {
            if (i == 0)
                coords.push(segments[i][0]);

            coords.push(segments[i][1]);
        }

        return GJ.createLineString(coords);
    }

    function degreesToRadians(degrees) {
        return degrees * Math.PI / 180;
    };

    function radiansToDegrees(radians) {
        return radians * 180 / Math.PI;
    };

})(Mapzania || {});