﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    var GJ = ns.GeoJson = ns.GeoJson || {};
    GJ.rotate = rotate;

    rotate.implemented = {
        Point: doPoint,
        MultiPoint: doMultiPoint,
        LineString: doLineString,
        MultiLineString: doMultiLineString,
        Polygon: doPolygon,
        MultiPolygon: doMultiPolygon,
        GeometryCollection: doGeometryCollection
    };

    return ns;


    //---------------------------------------------------------
    function rotate(geometry, origin, angle) {

        var type = GJ.getType(geometry);
        var func = rotate.implemented[type];

        if (!func)
            GJ.Exceptions.notImplemented(type, "rotate");

        return func(geometry, origin, angle);
    }

    //---------------------------------------------------------

    function doPoint(point, origin, angle) {

        var pointX = point.coordinates[0];
        var pointY = point.coordinates[1];
        var originX = origin.coordinates[0];
        var originY = origin.coordinates[1];

        var cos = Math.cos(angle);
        var sin = Math.sin(angle);

        // Rotate the point (counter clockwise)
        var x = (pointX - originX) * cos - (pointY - originY) * sin + originX;
        var y = (pointX - originX) * sin + (pointY - originY) * cos + originY;

        return GJ.createPoint(x, y);
    }

    function doMultiPoint(mpoint, origin, angle) {

        var res = GJ.createMultiPoint([]);

        mpoint.coordinates.forEach(function (coord) {
            var point = GJ.createPoint(coord);
            var rotated = doPoint(point, origin, angle);
            res.coordinates.push(rotated.coordinates);
        })

        return res;
    }

    function doLineString(line, origin, angle) {

        var res = GJ.createLineString([]);

        line.coordinates.forEach(function (coord) {
            var point = GJ.createPoint(coord);
            var rotated = doPoint(point, origin, angle);
            res.coordinates.push(rotated.coordinates);
        })

        return res;
    }

    function doMultiLineString(mline, origin, angle) {

        var res = GJ.createMultiLineString([]);

        mline.coordinates.forEach(function (coord) {
            var line = GJ.createLineString(coord);
            var rotated = doLineString(line, origin, angle);
            res.coordinates.push(rotated.coordinates);
        })

        return res;
    }

    function doPolygon(poly, origin, angle) {
        var res = GJ.createPolygon([]);

        poly.coordinates.forEach(function (coord) {
            var line = GJ.createLineString(coord);
            var rotated = doLineString(line, origin, angle);
            res.coordinates.push(rotated.coordinates);
        })

        return res;
    }

    function doMultiPolygon(mpoly, origin, angle) {
        var res = GJ.createMultiPolygon([]);

        mpoly.coordinates.forEach(function (coord) {
            var poly = GJ.createPolygon(coord);
            var rotated = doPolygon(poly, origin, angle);
            res.coordinates.push(rotated.coordinates);
        })

        return res;
    }

    function doGeometryCollection(gc, origin, angle) {
        var res = {
            type: "GeometryCollection",
            geometries:[]
        }

        gc.geometries.forEach(function (geom) {
            var rotated = rotate(geom, origin, angle);
            res.geometries.push(rotated);
        })

        return res;
    }


})(Mapzania || {});