﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    var GJ = ns.GeoJson = ns.GeoJson || {};
    GJ.slice = slice;

    slice.implemented = {
        LineString: {
            //LineString: doLineStringWithLineString,
            Point: doLineStringWithPoint
        }
    }

    return ns;

    //---------------------------------------------------------

    function slice(geometry, splitBy) {

        var type = GJ.getType(geometry);

        var funcs = slice.implemented[type];

        if (!funcs)
            GJ.Exceptions.notImplemented(type, "slice");

        type = GJ.getType(splitBy);

        var func = funcs[type];

        if (!func)
            GJ.Exceptions.notImplemented(type, "slice");

        return func(geometry, splitBy);
    }

    //---------------------------------------------------------

    function doLineStringWithLineString(line1, line2) {
    }

    function doLineStringWithPoint(line, point) {
        var segments = GJ.Math.segmentize(line);

        var calcs = [];

        for (var i = 0; i < segments.length; i++) {
            calcs.push(_calc(point.coordinates, segments[i]));
        }

        var min = calcs.reduce(function (prev, current) {
            return (current.distance < prev.distance) ? current : prev;
        });

        var idx = calcs.indexOf(min);

        var segments1 = segments.slice(0, idx);
        var segments2 = segments.slice(idx + 1);

        segments1.push([segments[idx][0], min.nearest]);
        segments2.unshift([min.nearest, segments[idx][1]]);

        return [
            GJ.Math.desegmentize(segments1),
            GJ.Math.desegmentize(segments2)
        ];
    }

    //---------------------------------------------------------

    function _calc(point, segment) {

        var start = segment[0];
        var end = segment[1];

        var nearest = GJ.Math.nearestPointOnSegment(point, start, end);
        var distance = GJ.Math.distance(point[0], point[1], nearest[0], nearest[1]);

        return {
            nearest: nearest,
            distance: distance
        };
    }

})(Mapzania || {});