﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    var GJ = ns.GeoJson = ns.GeoJson || {};
    GJ.snapTo = snapTo;

    snapTo.implemented = {
        LineString: doLineString,
        MultiLineString: doMultiLineString,
        //Polygon: doPolygon,
        //MultiPolygon: doMultiPolygon
    };

    return ns;

    //---------------------------------------------------------

    function snapTo(point, geometry) {

        var type = GJ.getType(geometry);
        var func = snapTo.implemented[type];

        if (!func)
            GJ.Exceptions.notImplemented(type, "snapTo");

        return func(point, geometry);
    }

    //---------------------------------------------------------

    function doLineString(point, line) {
        var result = _nearestResult(point, line.coordinates);

        return GJ.createPoint(result.coords);
    }

    function doMultiLineString(point, mline) {
        var coords = mline.coordinates;

        var currentDistance = Number.MAX_VALUE;
        var currentCoords = null;

        for (var i = 0; i < coords.length; i++) {
            var res = _nearestResult(point, coords[i]);

            if (res.distance < currentDistance) {
                currentDistance = res.distance;
                currentCoords = res.coords;
            }
        }

        return GJ.createPoint(currentCoords);
    }

    //function doPolygon(point, mline) {
    //}

    //function doMultiPolygon(point, mline) {
    //}

    //---------------------------------------------------------

    function _nearestResult(point, coords) {

        var pnt = point.coordinates;

        var currentDistance = Number.MAX_VALUE;
        var currentCoords = null;

        for (var i = 1; i < coords.length; i++) {
            var start = coords[i - 1];
            var end = coords[i];

            var nearest = GJ.Math.nearestPointOnSegment(pnt, start, end);
            var distance = GJ.Math.distance(pnt[0], pnt[1], nearest[0], nearest[1]);

            if (distance < currentDistance) {
                currentCoords = nearest;
                currentDistance = distance;
            }
        }

        return {
            coords: currentCoords,
            distance: currentDistance
        };
    }

})(Mapzania || {});