﻿/// <reference path="../../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    ns.LabelEngine = LabelEngine;
    return ns;

    //---------------------------------------------------------

    function LabelEngine(layerManager, view) {

        this.run = function () {

            var state = view.getState();
            var trf = view.getPixelTransformer();

            var allLabels = [];

            var layers = layerManager.getLayers();

            layers.sort(function (x, y) {
                return x.ordinal - y.ordinal;
            });

            layers.forEach(function (layer) {

                if (layer.getType() == "VectorLayer" && layer.isVisible()) {

                    var theme = layer.getThemer().getTheme(state.zoom);

                    if (theme) {

                        var features = layer
                            .getFeatureEngine()
                            .getLoadedFeatureCollection()
                            .features;

                        var labels = new ns.LabelMaker()
                            .makeLabels(features, theme, state);

                        labels.sort(function (x, y) {
                            return y.weight - x.weight;
                        });

                        ns.LabelFilters.removeDuplicates(labels);
                        ns.LabelFilters.removeCollisions(labels, trf);

                        ns.Util.arrayAppend(labels, allLabels);
                    }
                }
            });

            //ns.LabelFilters.removeCollisions(allLabels, trf);

            return _toFeatureCollection(allLabels);
        }

        //---------------------------------------------------------

        function _toFeatureCollection(labels) {
            return {
                type: "FeatureCollection",
                features: labels.map(function (m) {
                    return {
                        type: "Feature",
                        geometry: m.centroid,
                        properties: {
                            text: m.text,
                            symbol: m.symbol,
                            rotation: m.rotation
                        }
                    }
                })
            };
        }
    }

})(Mapzania || {});