﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    ns.LabelFilters = {
        removeDuplicates: removeDuplicates,
        removeCollisions: removeCollisions
    }

    var GJ = ns.GeoJson;

    return ns;

    //---------------------------------------------------------

    function removeDuplicates(labels) {

        var labelsToRemove = {};

        for (var i = labels.length - 1; i >= 0; i--) {

            var symbol = labels[i].symbol;

            if (symbol && symbol.allowDuplicates)
                continue;

            var text = labels[i].text;

            if (labelsToRemove[text])
                labels.splice(i, 1);

            labelsToRemove[text] = true;
        }
    }

    function removeCollisions(labels, trf) {

        for (var i = 0; i < labels.length; i++) {
            _boundarize(labels[i], trf);
        }

        for (var i = labels.length - 1; i >= 0; i--) {

            var symbol = labels[i].symbol;

            if (symbol && !symbol.collisionDetection !== false)
                continue;

            var collides = false;

            for (var j = 0; j < labels.length; j++) {
                if (labels[j] == labels[i]) continue;

                if (GJ.intersects(labels[i].boundary, labels[j].boundary)) {
                    collides = true;
                    break;
                }
            }

            if (collides)
                labels.splice(i, 1);
        }
    }

    //---------------------------------------------------------

    function _boundarize(label, trf) {

        var size = trf.measureText(label.text, label.symbol);

        var pos = trf.translate(label.centroid);

        var bbox = [
            pos.x - size.width / 2,
            pos.y - size.height / 2,
            pos.x + size.width / 2,
            pos.y + size.height / 2
        ];

        label.boundary = bbox;

        if (label.rotation != 0 || label.symbol.autoRotate) {
            var poly = GJ.createPolygon(bbox);
            poly = GJ.rotate(poly, GJ.createPoint(pos.x, pos.y), label.rotation);
            label.boundary = poly;
        }
    }

})(Mapzania || {});