﻿/// <reference path="../../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    ns.LabelMaker = LabelMaker;
    return ns;

    //---------------------------------------------------------

    function LabelMaker() {

        this.makeLabels = function (features, theme, state) {

            var labels = [];

            features.forEach(function (ftr) {

                var label = _getLabel(theme, ftr, state);

                if (label)
                    labels.push(label);
            })

            return labels;
        }

        //---------------------------------------------------------

        function _getLabel(theme, feature, state) {

            var result = {
                text: theme.getLabelText(feature),
                symbol: theme.getLabelSymbol(feature),
                rotation: 0,
                weight: 0
            };

            if (result.text == null)
                return null;

            if (result.symbol == null)
                return null;

            switch (feature.geometry.type) {
                case "LineString":
                case "MultiLineString":
                    var info = _getLineInfo(result.symbol, feature.geometry, state);
                    if (!info)
                        return null;
                    ns.Util.assign(result, info);
                    break;
                case "Polygon":
                case "MultiPolygon":
                    result.weight = ns.GeoJson.getArea(feature.geometry);
                case "Point":
                case "MultiPoint":
                    result.centroid = ns.GeoJson.getCentroid(feature.geometry);
            }

            return result;
        }

        function _getLineInfo(symbol, geometry, state) {

            var cropped = ns.GeoJson.crop(geometry, state.bounds);

            if (!cropped)
                return null;

            var result = {
                rotation: 0,
                centroid: ns.GeoJson.getCentroid(cropped),
                weight: ns.GeoJson.getLength(cropped)
            }

            if (symbol.rotation != 0)
                result.rotation = symbol.rotation;

            if (symbol.autoRotate) {
                var ftr = ns.GeoJson.getFeatureAt(cropped, result.weight / 2);

                var rotation = 0;

                if (ftr)
                    rotation = ftr.properties.rotation;

                result.rotation += rotation;
            }

            //Ensure the labels do not show upside down
            if (result.rotation > Math.PI / 2)
                result.rotation -= Math.PI;

            if (result.rotation < -Math.PI / 2)
                result.rotation += Math.PI;

            return result;
        }
    }

})(Mapzania || {});