﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    ns.VectorLayer = VectorLayer;
    return ns;

    //---------------------------------------------------------

    function VectorLayer(layerJSON, featureEngine, themer, attribution) {

        var hidden = [];
        var filters = [];
        var responseFilters = [];
        var isVisible = layerJSON.visible;

        // ---INFO---

        this.getType = function () {
            return "VectorLayer";
        }

        this.getKey = function () {
            return layerJSON.key;
        }

        this.getFeatureEngine = function () {
            return featureEngine;
        }

        this.getThemer = function () {
            return themer;
        }

        this.getFilterText = function () {
            return layerJSON.filter;
        }

        this.getFields = function () {
            var fields = layerJSON.fields;

            if (ns.Util.isArray(fields))
                return [].concat(fields);

            return [];
        }

        this.isVisible = function () {
            return isVisible;
        }

        // ---BEHAVIOUR---

        this.show = function () {
            isVisible = true;
        }

        this.hide = function () {
            isVisible = false;
        }

        // ---FILTERS---

        this.getFilters = function () {
            return [].concat(filters);
        }

        this.addFilter = function (filter, index) {
            if (index)
                filters.splice(index, 0, filter)
            else
                filters.push(filter);
        }

        this.removeFilter = function (index) {
            filters.splice(index, 1);
        }

        this.clearFilters = function (layerKey) {
            filters = [];
        }

        // ---STYLES---

        this.addStyle = function (style) {
            var theme = new ns.DynamicTheme(style);

            var idx = themer.indexOfThemeType("DynamicTheme");

            if (idx > -1)
                themer.setTheme(idx, theme);
            else
                themer.addTheme(theme);
        }

        this.clearStyles = function () {
            var idx = themer.indexOfThemeType("DynamicTheme");

            if (idx > -1)
                themer.removeStyle(idx);
        }

        this.setFeatureStyle = function (id, style) {
            themer.setFeatureStyle(id, style);
        }

        this.clearFeatureStyle = function (layerKey, id) {
            themer.setFeatureStyle(id);
        }

        // ---FEATURES---

        this.addFeatureCollection = function (featureCollection) {
            featureEngine.addLocalFeatureCollection(featureCollection);
        }

        this.addFeature = function (feature) {
            var fc = ns.GeoJson.createFeatureCollection(feature);
            featureEngine.addLocalFeatureCollection(fc);
        }

        this.clearFeatures = function () {
            featureEngine.clearLocalFeatures();
        }

        this.hideFeature = function (id) {
            featureEngine.hideFeature(id);
        }

        this.showFeature = function (id) {
            featureEngine.showFeature(id);
        }

        // ---OTHER---

        this.getAttribution = function () {
            return attribution;
        }

        this.toJSON = function () {
            return {
                key: layerJSON.key,
                name: layerJSON.name,
                sourceKey: layerJSON.sourceKey,
                ordinal: layerJSON.ordinal,
                visible: isVisible
            };
        }

        //---------------------------------------------------------

        function _notifyClientFeaturesChanged(layer) {
            var data = {
                layerKey: layer.key,
                features: layer.localFeatures
            };

            me.fire("client_features_changed", data);
        }
    }

})(Mapzania || {})
