﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    ns.Leaflet = ns.Leaflet || {};

    ns.Leaflet.Layers = Layers;
    return ns;

    //---------------------------------------------------------

    function Layers(map) {

        var vlayers = [];
        var rlayers = [];
        var panes = {};

        ns.Log.debug("Object created.", "Leaflet.Layers", map);

        //-----------------------------------------------------

        this.addVectorLayer = function (layer, index) {

            var key = layer.getKey();

            //var pane = panes[key];

            //if (!pane) {
                var pane = map.createPane(key);
            //    panes[key] = pane;
            //}

            var zIndex = 499 - vlayers.length;
            pane.style.zIndex = zIndex;

            var leafletLayer = L.geoJSON(null, {
                pane: key,
                attribution: layer.getAttribution(),
                _mz: {
                    pane: pane,
                    key: key,
                    layer: layer,
                    zIndex: zIndex
                },

                style: function (feature) {
                    return ns.Leaflet.SymbolConverter.convert(feature.symbol);
                },

                pointToLayer: function (feature, latlng) {
                    var options = ns.Leaflet.SymbolConverter.convert(feature.symbol);

                    Object.assign(options, {
                        pane: pane,
                        map: map
                    });

                    return new ns.Leaflet.MarkerLayer(latlng, options);
                }
            })

            vlayers.push(leafletLayer);

            if (!Mapzania.Util.isUndefined(index)) {

                var lyr = this.at(vlayers.length - 1);

                for (var i = vlayers.length - 1; i > index; i--) {
                    //var lyr = this.at(i);
                    lyr.bringForwards();
                }
            }

            leafletLayer.addTo(map);
        }

        this.addRasterLayer = function (layer, index) {

            var options = {
                _mz: {
                    key: layer.getKey(),
                    layer: layer
                },

                attribution: layer.getAttribution()
            };

            var leafletLayer = layer.isGreyScale() ?
                new ns.Leaflet.GreyScaleTileLayer(layer.getUrl(), options) :
                L.tileLayer(layer.getUrl(), options);

            if (Mapzania.Util.isUndefined(index))
                rlayers.push(leafletLayer);
            else
                rlayers.splice(index, 0, leafletLayer);

            if (layer.isVisible()) {
                leafletLayer.addTo(map);
            }

            //legend.addBaseLayer(leafletLayer, layer.getKey());
        };

        //-----------------------------------------------------

        this.get = function (key) {
            var layer = vlayers.find(m => m.options._mz.key == key)
            return new Layer(map, this, layer);
        }

        this.at = function (index) {
            var layer = vlayers[index];
            return new Layer(map, this, layer);
        }

        this.sendBackwards = function (index) {
            var itm = vlayers[index];
            vlayers[index] = vlayers[index + 1];
            vlayers[index + 1] = itm;
        }

        this.bringForwards = function (index) {
            var itm = vlayers[index];
            vlayers[index] = vlayers[index - 1];
            vlayers[index - 1] = itm;
        }

        this.indexOf = function (layer) {
            return vlayers.indexOf(layer);
        }

        this.remove = function (key) {

            var layer = vlayers.find(m => m.options._mz.key == key);

            if (layer) {

                var idx = this.indexOf(layer);

                vlayers.splice(idx, 1);

                if (map.hasLayer(layer)) {
                    map.removeLayer(layer);

                    delete map.getPanes()[key];
                    map._paneRenderers = {};

                    var paneKey = "leaflet-" + key + "-pane";

                    var el = document.getElementsByClassName(paneKey)[0];
                    el.parentNode.removeChild(el);
                }
            }

            layer = rlayers.find(m => m.options._mz.key == key);

            if (layer) {

                var idx = this.indexOf(layer);

                rlayers.splice(idx, 1);

                if (map.hasLayer(layer))
                    map.removeLayer(layer);                
            }
        }

        //-----------------------------------------------------

        this.createLabelLayer = function () {
            var key = "MZ_LABELS";
            var index = 625;

            var pane = map.createPane(key);
            pane.style.zIndex = index;

            var layer = new ns.Leaflet.LabelLayer({ pane: key });
            layer.addTo(map);

            return layer;
        }

        this.createSelectorLayer = function () {
            var key = "MZ_SELECTOR";
            var index = 800;

            var pane = map.createPane(key);
            pane.style.zIndex = index;

            var layer = L.geoJSON(null, {
                pane: key,

                style: function (feature) {
                    return {
                        color: "#000000",
                        fillColor: "#FFFF00",
                        weight: 2,
                        radius: 5
                    }
                },

                pointToLayer: function (feature, latlng) {
                    return L.circleMarker(latlng, { pane: key });
                }
            });

            layer.addTo(map);

            return layer;
        }
    }

    function Layer(map, layers, layer) {

        this.show = function () {
            if (!map.hasLayer(layer))
                map.addLayer(layer);
        }

        this.hide = function () {
            if (map.hasLayer(layer))
                map.removeLayer(layer);
        }

        this.clear = function () {
            layer.clearLayers();
        }

        this.add = function (data) {
            layer.addData(data);
        }

        this.getIndex = function () {
            return layers.indexOf(layer);
        }

        this.bringForwards = function () {
            var idx = this.getIndex();

            var from = layers.at(idx);
            var to = layers.at(idx - 1);

            layers.bringForwards(idx);

            from.setZ(idx - 1);
            to.setZ(idx);
        }

        this.sendBackwards = function () {

            var idx = this.getIndex();

            var from = layers.at(idx);
            var to = layers.at(idx + 1);

            layers.sendBackwards(idx);

            from.setZ(idx + 1);
            to.setZ(idx);
        }

        this.setZ = function (index) {

            var z = 499 - index;

            var mz = layer.options._mz;
            mz.zIndex = z;

            var paneKey = "leaflet-" + mz.key + "-pane";
            var el = document.getElementsByClassName(paneKey)[0];
            el.style.zIndex = z;
        }
    }

})(Mapzania || {});