﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    ns.Leaflet = ns.Leaflet || {};
    ns.Leaflet.MarkerLayer = extendClass();

    register();

    return ns;

    //---------------------------------------------------------

    function register() {
        L.SVG.include({
            mapzania_update: function (layer) {

                var p = layer._point;
                var s = layer._radius;
                var shape = layer.options.pattern;

                if (shape === "diamond") {
                    var d = "M " + (p.x - s) + " " + (p.y) + ", L " + (p.x) + " " + (p.y - s) + ", L" + (p.x + s) + " " + (p.y) + ", L" + (p.x) + " " + (p.y + s) + ", L" + (p.x - s) + " " + (p.y);
                    this._setPath(layer, d);
                }
                if (shape === "square") {
                    var d = "M " + (p.x - s) + " " + (p.y - s) + ", L " + (p.x + s) + " " + (p.y - s) + ", L" + (p.x + s) + " " + (p.y + s) + ", L" + (p.x - s) + " " + (p.y + s) + ", L" + (p.x - s) + " " + (p.y - s);
                    this._setPath(layer, d);
                }
                if (shape === "triangle") {
                    var d = "M" + (p.x - s) + " " + (p.y + s) + " L" + (p.x) + " " + (p.y - s) + " L" + (p.x + s) + " " + (p.y + s) + " Z";
                    this._setPath(layer, d);
                }
                if (shape === "circle") {
                    this._updateCircle(layer)
                }
                if (shape === "cross") {
                    s = s / 2;
                    var d = 'M' + (p.x + s) + ',' + (p.y + s) +
                        'L' + (p.x - s) + ',' + (p.y - s) +
                        'M' + (p.x - s) + ',' + (p.y + s) +
                        'L' + (p.x + s) + ',' + (p.y - s);
                    this._setPath(layer, d);
                }
            }
        });

        L.SVG.include({
            _updateStyle: function (layer) {
                var path = layer._path,
                    options = layer.options;

                if (!path) { return; }

                if (options.stroke) {
                    path.setAttribute('stroke', options.color);
                    path.setAttribute('stroke-opacity', options.opacity);
                    path.setAttribute('stroke-width', options.weight);
                    path.setAttribute('stroke-linecap', options.lineCap);
                    path.setAttribute('stroke-linejoin', options.lineJoin);

                    if (options.dashArray) {
                        path.setAttribute('stroke-dasharray', options.dashArray);
                    } else {
                        path.removeAttribute('stroke-dasharray');
                    }

                    if (options.dashOffset) {
                        path.setAttribute('stroke-dashoffset', options.dashOffset);
                    } else {
                        path.removeAttribute('stroke-dashoffset');
                    }
                } else {
                    path.setAttribute('stroke', 'none');
                }

                if (options.fill) {
                    path.setAttribute('fill', options.fillColor || options.color);
                    path.setAttribute('fill-opacity', options.fillOpacity);
                    path.setAttribute('fill-rule', options.fillRule || 'evenodd');
                } else {
                    path.setAttribute('fill', 'none');
                }

                if (options.transform) {
                    path.setAttribute("transform", options.transform);
                }
            }
        });

    }

    function extendClass() {
        //return L.CircleMarker.extend({
        //});

        var layerClass = L.Path.extend({
            options: {
                fill: true,
                shape: 'square',
                radius: 10
            },

            initialize: function (latlng, options) {
                L.setOptions(this, options);
                this._latlng = L.latLng(latlng);
                this._radius = this.options.radius;
            },

            setLatLng: function (latlng) {
                this._latlng = L.latLng(latlng);
                this.redraw();
                return this.fire('move', { latlng: this._latlng });
            },

            getLatLng: function () {
                return this._latlng;
            },

            setRadius: function (radius) {
                this.options.radius = this._radius = radius;
                return this.redraw();
            },

            getRadius: function () {
                return this._radius;
            },

            setTransform: function () {
                var rotation = this.options.rotation;
                var offset = this.options.offset;

                var transforms = [];

                if (offset) {
                    transforms.push("translate(" + offset.x + " " + offset.y + ")");
                }                

                if (rotation) {
                    var point = this.options.map.latLngToLayerPoint(this._latlng);

                    transforms.push("rotate(" +
                        rotation.angle + " " +
                        (point.x - rotation.originX) + " " +
                        (point.y - rotation.originY) + ")");
                }

                if (transforms.length > 0) {
                    this.options.transform = transforms.join(" ");
                }
            },

            setStyle: function (options) {
                var radius = options && options.radius || this._radius;
                L.Path.prototype.setStyle.call(this, options);
                this.setRadius(radius);
                this.setTransform();
                return this;
            },

            _project: function () {
                this._point = this._map.latLngToLayerPoint(this._latlng);
                this._updateBounds();
            },

            _updateBounds: function () {
                var r = this._radius,
                    r2 = this._radiusY || r,
                    w = this._clickTolerance(),
                    p = [r + w, r2 + w];
                this._pxBounds = new L.Bounds(this._point.subtract(p), this._point.add(p));
            },

            _update: function () {
                if (this._map) {
                    this._updatePath();
                }
            },

            _updatePath: function () {
                this._renderer.mapzania_update(this);
            },

            _empty: function () {
                return this._size && !this._renderer._bounds.intersects(this._pxBounds);
            },

            toGeoJSON: function () {
                return L.GeoJSON.getFeature(this, {
                    type: 'Point',
                    coordinates: L.GeoJSON.latLngToCoords(this.getLatLng())
                });
            }

        });

        return layerClass;
    }

})(Mapzania || {});



