﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------
    ns.Leaflet = ns.Leaflet || {};
    ns.Leaflet.SymbolConverter = {
        convert: convert,
        toCss: toCss
    }
    return ns;

    //---------------------------------------------------------

    function convert(symbol) {

        if (!symbol)
            ns.Util.logAndThrow("Null or undefined symbol.", "Leaflet.SymbolConverter.convert()");

        switch (symbol.type) {
            case "PointSymbol":
                return _convertPointSymbol(symbol);
            case "CompoundPointSymbol":
                return _convertPointSymbol(symbol.points[symbol.points.length - 1]);
            case "LineSymbol":
                return _convertLineSymbol(symbol);
            case "CompoundLineSymbol":
                return _convertLineSymbol(symbol.lines[symbol.lines.length - 1]);
            case "FillSymbol":
                return _convertPolygonSymbol(symbol);
            case "Labeller":
                return _convertLabelSymbol(symbol);
            default:
                ns.Util.logAndThrow("Unknown SymbolType=" + symbol.type, "Leaflet.SymbolConverter.convert()");
        }
    }

    function toCss(symbol) {
        if (!symbol)
            ns.Util.logAndThrow("Null or undefined symbol.", "Leaflet.SymbolConverter.toCss()");

        switch (symbol.type) {
            case "Labeller":
                return _convertLabelSymbolToCss(symbol);
            default:
                ns.Util.logAndThrow("Unsupported SymbolType=" + symbol.type, "Leaflet.SymbolConverter.toCss()");
        }
    }

    //---------------------------------------------------------

    function _convertPointSymbol(symbol) {

        var patterns = ["circle", "square", "triangle", "cross", "diamond", "crosshair", "chevron"];

        var result = {
            pattern: patterns[symbol.pattern],
            fillColor: symbol.color,
            fillOpacity: symbol.opacity / 100,
            radius: symbol.size,
            stroke: false,
            rotation: symbol.rotation,
            offset: symbol.offset
        }

        if (symbol.border && symbol.border.type == "LineSymbol") {

            var bdr = symbol.border;

            result.stroke = true;
            result.color = bdr.color;
            result.weight = bdr.size;
            result.opacity = bdr.opacity / 100;
        }

        return result;
    }

    function _convertLineSymbol(symbol) {

        //var patterns = ["solid", "dash", "dots"];
        var patterns = ["solid", "10,5", "1,3"];
        var endcaps = ["round", "butt", "square"];
        var joins = ["round", "bevel", "mitre"];

        return {
            color: symbol.color,
            opacity: symbol.opacity / 100,
            weight: symbol.size,
            lineCap: symbol.endCap ? endcaps[symbol.endCap] : endcaps[0],
            lineJoin: symbol.join ? joins[symbol.join] : joins[0],
            dashArray: symbol.pattern ? patterns[symbol.pattern] : patterns[0]
        }
    }

    function _convertPolygonSymbol(symbol) {

        //var patterns = ["solid", "dash", "dots"];
        var patterns = ["solid", "10,5", "1,3"];
        var endcaps = ["round", "butt", "square"];
        var joins = ["round", "bevel", "mitre"];

        var result = {
            fillColor: symbol.color,
            fillOpacity: symbol.opacity / 100,
            stroke: false
        }

        if (symbol.outline && symbol.outline.type == "LineSymbol") {

            var ol = symbol.outline;

            result.stroke = true;
            result.color = ol.color;
            result.weight = ol.size;
            result.opacity = ol.opacity / 100;
            result.lineCap = ol.endCap ? endcaps[ol.endCap] : endcaps[0];
            result.lineJoin = ol.join ? joins[ol.join] : joins[0];
            result.dashArray = ol.pattern ? patterns[ol.pattern] : patterns[0];
        }

        return result;
    }

    function _convertLabelSymbol(symbol) {
        return symbol;
    }

    function _convertLabelSymbolToCss(symbol) {

        var values = {
            "position": "absolute",
            "white-space": "nowrap",
            "color": symbol.color,
            "font-size": symbol.fontSize + "pt",
            "font-family": symbol.fontFamily,
            "opacity": symbol.opacity / 100,
            "padding": "0px",
            //"display":"inline",
            "cursor": "inherit"
        };

        if (symbol.bold)
            values["font-weight"] = "bold";

        if (symbol.italic)
            values["font-style"] = "italic";

        if (symbol.haloThickness) {
            var shadow = "0 0 " + symbol.haloThickness + "pt " + symbol.haloColor;
            values["text-shadow"] = shadow + "," + shadow + "," + shadow;
        }

        var res = "";
        for (key in values) {
            res += key + ":" + values[key] + ";";
        }

        return res;
    }

})(Mapzania || {});