﻿/// <reference path="../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------
    ns.LayerThemer = LayerThemer;
    return ns;

    //---------------------------------------------------------

    function LayerThemer() {

        var themes = [];
        var featureStyles = {};
        var isClean = true;

        themes.push(new ns.DefaultTheme());

        ns.Log.debug("Object created.", "LayerThemer");

        //-------------------MANAGEMENT------------------------

        this.addTheme = function (theme) {
            //remove default theme
            if (isClean) {
                themes = [];
                isClean = false;
            }

            themes.push(theme);
        }

        this.addThemeJson = function (themeJson) {
            //remove default theme
            if (isClean) {
                themes = [];
                isClean = false;
            }

            themes.push(_createTheme(themeJson));
        }

        this.updateTheme = function (index, themeJson) {
            themes[index] = _createTheme(themeJson);
        }

        this.removeTheme = function (index) {
            themes.splice(index, 1);

            if (themes.length == 0)
                themes.push(new ns.DefaultTheme());
        }

        this.indexOfThemeType = function (type) {
            return themes.findIndex(function (m) {
                m.getType() == type;
            });
        }

        this.setFeatureStyle = function (id, style) {
            featureStyles[id] = style;
        }

        this.clearFeatureStyle = function (id) {
            delete featureStyles[id];
        }

        //-------------------QUERY-----------------------------

        this.getTheme = function (zoom, id) {

            var check = ns.Util.isNullOrUndefined;

            // Does single feature styling apply?
            if (!check(id) && !check(featureStyles[id]))
                return new ns.DynamicTheme(featureStyles[id]);

            for (var i = themes.length - 1; i >= 0; i--) {

                var theme = themes[i];
                if (theme.getType() == "DynamicTheme")
                    return theme;
            }

            for (var i = themes.length - 1; i >= 0; i--) {

                var theme = themes[i];

                var minZoom = theme.getMinZoom();
                var maxZoom = theme.getMaxZoom();

                if (zoom >= minZoom && zoom <= maxZoom)
                    return theme;
            }

            return null;
        }

        this.getFilterText = function (zoom) {

            var theme = this.getTheme(zoom);

            if (theme)
                return theme.getFilterText();

            return null;
        }

        //----------------------------------------------------

        function _createTheme(themeJson) {
            if (themeJson.type == "StandardTheme")
                return new ns.StandardTheme(themeJson);

            if (themeJson.type == "RangeTheme")
                return new ns.RangeTheme(themeJson);

            if (themeJson.type == "Theme")
                return new ns.Theme(themeJson);

            else
                ns.Util.logAndThrow("Unknown theme type=" + themeJson.type, "LayerThemer._createTheme()");
        }
    }

})(Mapzania || {})
