﻿/// <reference path="../../mapzania.js"/>

var Mapzania = (function (ns) {

    //---------------------------------------------------------

    ns.Symbols = {

        _defaultPoint: _defaultPoint(),
        _defaultLine: _defaultLine(),
        _defaultPolygon: _defaultPolygon(),
        _defaultLabel: _defaultLabel(),
        createPoint: createPoint,
        createLine: createLine,
        createPolygon: createPolygon,
        createLabel: createLabel
    }

    return ns;

    //---------------------------------------------------------

    function createPoint(style) {
        return {
            type: "PointSymbol",
            pattern: 0,
            size: style.size,
            color: style.fillColor,
            opacity: style.fillOpacity * 100,
            border: {
                type: "LineSymbol",
                color: style.strokeColor,
                size: style.strokeWidth,
                opacity: style.strokeOpacity * 100
            }
        }
    }

    function createLine(style) {
        return {
            type: "LineSymbol",
            color: style.strokeColor,
            size: style.strokeWidth,
            opacity: style.strokeOpacity * 100
        };
    }

    function createPolygon(style) {
        return {
            type: "FillSymbol",
            color: style.fillColor,
            opacity: style.fillOpacity * 100,
            outline: {
                type: "LineSymbol",
                color: style.strokeColor,
                size: style.strokeWidth,
                opacity: style.strokeOpacity * 100
            }
        }
    }

    function createLabel(label) {

        var isBold = label.fontWeight && label.fontWeight.toLowerCase() == "bold";

        var res = {
            type: "LabelSymbol",
            color: label.fill,
            bold: isBold,
            fontSize: 10,
            rotation: 0,
            fontFamily: "arial",
            offset: { x: 0, y: 0 },
            field: label.field
        };

        return res;
    }

    //---------------------------------------------------------

    function _defaultPoint() {
        return {
            "type": "CompoundPointSymbol",
            "drawByLayer": true,
            "points": [
                {
                    "type": "PointSymbol",
                    "pattern": 0,
                    "size": 3.0,
                    "color": "#000000",
                }
            ],
            "opacity": 100
        };
    }

    function _defaultLine() {
        return {
            "type": "CompoundLineSymbol",
            "drawByLayer": true,
            "lines": [
                {
                    "type": "LineSymbol",
                    "pattern": 0,
                    "color": "#000000",
                    "size": 1.0,
                }
            ],
            "opacity": 100
        };
    }

    function _defaultPolygon() {
        return {
            "type": "FillSymbol",
            "pattern": 0,
            "color": "rgba(0, 0, 0, 0.2)",
            "outline": {
                "type": "CompoundLineSymbol",
                "drawByLayer": true,
                "lines": [
                    {
                        "type": "LineSymbol",
                        "pattern": 0,
                        "color": "#000000",
                        "size": 1.0
                    }
                ],
                "opacity": 100
            },
            "opacity": 100
        }
    }

    function _defaultLabel() {
        return {
            type: "Labeller",
            color: "#000000",
            bold: false,
            italic: false,
            fontSize: 10,
            fontFamily: "arial",
            rotation: 0,
            //haloColor: "#FFFFFF",
            //haloThickness: 1.6,
            //offset: { x: 0, y: -12 },
            //opacity: 100
        };
    }

})(Mapzania || {})
