﻿(function () {
    "use strict";

    angular
        .module("app")
        .controller("VectorLayerController", controller);

    controller.$inject = [
        "$scope",
        "ThemeService",
        "ContextService",
        "CommandService"
    ];

    //-------------------------------------------------

    function controller(
        $scope,
        ThemeService,
        ContextService,
        CommandService
    ) {

        var vm = this;

        this.addTheme = addTheme;
        this.selectTheme = selectTheme;

        init();

        //---------------------------------------------

        function init() {

            ContextService.on("context_set", (data) => {
                if (data.type == "VectorLayer") {
                    bind();
                }
            });

            bind();
        }

        function bind() {

            if (vm.model)
                unwatch(vm.model);

            var model = ContextService.getCurrent();

            var i = 0;

            vm.model = {
                visible: model.visible,
                key: model.key,
                name: model.name,
                sourceKey: model.sourceKey,
                themes: model.themes.map(m => {
                    return {
                        index: i++,
                        text: m.toString()
                    };
                })
            };
        
            watch(vm.model, onChanged);

            function onChanged(prop, action, newVal, oldVal) {

                if (prop == "visible") {
                    CommandService.changeLayerVisibility(model, newVal, oldVal);
                } else {
                    CommandService.change(model, prop, newVal, oldVal);
                }

                $scope.$apply();
            };
        }

        function addTheme() {

            var layer = ContextService.getCurrent();
            var layerKey = layer.key;

            ThemeService.createTheme(layer)
                .then(theme => {
                    CommandService.addTheme(layer, theme);
                    ContextService.setContext(theme, theme.toString(), 2);
                });
        }

        function selectTheme(index) {
            var layer = ContextService.getCurrent();
            var theme = layer.themes[index];
            ContextService.setContext(theme, theme.toString(), 2);
        }
    }

})();