﻿(function () {
    "use strict";

    angular
        .module("app")
        .service("Application", service);

    service.$inject = [];

    //-------------------------------------------------

    function service() {

        var svc = this;
        svc.init = init;

        //---------------------------------------------

        function init(schema, view) {
            svc.model = new Model(schema);
            svc.view = view;
        }

        //---------------------------------------------

        function Model(schema) {

            this.getStartExtent = function () {
                return Object.assign({}, schema.startExtent);
            }

            this.getMap = function () {
                return schema;
            }

            this.getStartExtent = function () {
                return schema.startExtent;
            }

            this.getVectorLayers = function () {
                return schema.layers.filter(m => m.type == "VectorLayer");
            }

            this.getRasterLayers = function () {
                return schema.layers.filter(m => m.type == "RasterLayer");
            }

            this.addLayer = function (layer, index) {
                switch (layer.type) {
                    case "VectorLayer":
                        schema.layers.splice(index, 0, layer);
                        break;
                    case "RasterLayer":
                        var cnt = this.getVectorLayers().length;
                        schema.layers.splice(index + cnt, 0, layer);
                        break
                    default:
                        throw "Invalid layer type";
                }

                console.log("LAYERS.......", schema.layers);
            }

            this.removeLayer = function (layer) {
                var idx = schema.layers.indexOf(layer);
                schema.layers.splice(idx, 1);

                console.log("LAYERS.......", schema.layers);
            }

            this.indexOfLayer = function (layer) {
                switch (layer.type) {
                    case "VectorLayer":
                        return schema.layers.indexOf(layer);
                    case "RasterLayer":
                        var cnt = this.getVectorLayers().length;
                        return schema.layers.indexOf(layer) - cnt;
                    default:
                        throw "Invalid layer type";
                }
            }

            this.sendLayerBackwards = function (layer) {

                var layers = schema.layers;

                var idx = layers.indexOf(layer);
                var temp = layers[idx + 1];

                layers[idx + 1] = layer;
                layers[idx] = temp;
            }

            this.bringLayerForwards = function (layer) {

                var layers = schema.layers;

                var idx = layers.indexOf(layer);
                var temp = layers[idx - 1];

                layers[idx - 1] = layer;
                layers[idx] = temp;
            }

            this.addTheme = function (layer, theme, index) {
                if (Mapzania.Util.isUndefined(index))
                    layer.themes.push(theme);
                else
                    layer.themes.splice(index, 0, theme);
            }

            this.removeTheme = function (layer, theme) {
                var idx = layer.themes.indexOf(theme);
                layer.themes.splice(idx, 1);
            }

            this.addThemeValue = function (theme, themeValue, index) {
                if (Mapzania.Util.isUndefined(index))
                    theme.values.push(themeValue);
                else
                    theme.values.splice(index, 0, themeValue);
            }

            this.removeThemeValue = function (theme, themeValue) {
                var idx = theme.values.indexOf(themeValue);
                theme.values.splice(idx, 1);
            }
        }
    }

})();