﻿(function () {
    "use strict";

    angular
        .module("app")
        .controller("ThemeController", controller);

    controller.$inject = [
        "$scope",
        "dialog",
        "ThemeService",
        "SourcesService",
        "SymbolService",
        "ContextService",
        "CommandService"
    ];

    //-------------------------------------------------

    function controller(
        $scope,
        dialog,
        ThemeService,
        SourcesService,
        SymbolService,
        ContextService,
        CommandService
    ) {
        var $util = Mapzania.Util;

        var vm = this;

        vm.symbolTypes = [];
        vm.symbologies = [];
        vm.symbolType = "CompoundPointSymbol";

        vm.noOfThemeValueClasses = 8;

        vm.fields = [];

        vm.deleteTheme = deleteTheme;

        vm.addThemeValue = addThemeValue;
        vm.deleteThemeValue = deleteThemeValue;
        vm.autoPopulateThemeValues = autoPopulateThemeValues;

        init();

        //---------------------------------------------

        function init() {

            vm.symbolTypes = SymbolService.getSymbolTypes();
            vm.symbologies = SymbolService.getSymbologies();

            var layer = ContextService.getParent();

            SourcesService.getSource(layer.sourceKey)
                .then(data => {
                    vm.fields = data.fields;
                });

            ContextService.on("context_set", (data) => {
                if (data.type == "Theme" || data.type == "ThemeValue") {
                    bind();
                }
            });

            bind();
        }

        function bind() {

            if (vm.model)
                unwatch(vm.model);

            var model = ContextService.getCurrent();
            var layerKey = ContextService.getParent().key;

            vm.model = Object.assign({}, model);
            vm.model.symbol = $util.assign({}, model.symbol);
            vm.model.labeller = $util.assign({}, model.labeller);
            delete vm.model.values;

            watch(vm.model, function (prop, action, newVal, oldVal) {
                CommandService.changeAndUpdate(model, prop, newVal, oldVal, layerKey);
                $scope.$apply();
            });

            if (vm.values)
                unwatch(vm.values);

            vm.values = model.values.map(m => $util.assign({}, m));

            watch(vm.values, function (prop, action, newVal, oldVal, obj) {
                if (action == "set") {
                    var idx = vm.values.indexOf(obj);
                    CommandService.changeAndUpdate(model.values[idx], prop, newVal, oldVal, layerKey);
                    //console.log("here");
                    $scope.$apply();
                }
            });
        }

        function deleteTheme() {

            dialog.confirm("Are you sure you want to delete this theme?")
                .then(m => {
                    var layer = ContextService.getParent();
                    var theme = ContextService.getCurrent();

                    CommandService.removeTheme(layer, theme);
                    ContextService.setContext(layer, layer.key, 1);
                });
        }

        function addThemeValue() {

            var layer = ContextService.getParent();
            var theme = ContextService.getCurrent();

            ThemeService.createThemeValue({ symbolType: theme.symbol.type })
                .then(themeValue => {

                    vm.values.push(themeValue);
                    CommandService.addThemeValue(layer, theme, themeValue);
                });
        }

        function deleteThemeValue(themeValue) {

            var layer = ContextService.getParent();
            var theme = ContextService.getCurrent();

            dialog.confirm("Are you sure you want to delete this value?")
                .then(() => {

                    var idx = vm.values.indexOf(themeValue);
                    vm.values.splice(idx, 1);

                    CommandService.removeThemeValue(layer, theme, themeValue);
                });
        }

        function autoPopulateThemeValues() {

            var layer = ContextService.getParent();
            var theme = ContextService.getCurrent();

            var options = {
                symbology: theme.symbology,
                sourceKey: layer.sourceKey,
                field: theme.field,
                noOfClasses: vm.noOfThemeValueClasses,
                symbolType: theme.symbol.type
            };

            ThemeService.createThemeValues(options)
                .then(values => {
                    CommandService.changeAndUpdate(theme, "values", values, theme.values, layer.key);
                    vm.values = values.map(m => Object.assign({}, m));
                });
        }
    }
})();

            //$scope.$watch(() => vm.symbolType, (newVal, oldVal) => {

            //    if (newVal == oldVal)
            //        return;

            //    dialog.confirm("Changing the symbol type will reset symbology, clear symbol values and reset the default symbol. Are you sure you want to proceeed?")

            //        .then(resolve => {

            //            vm.model.values = [];
            //            vm._changeQueued = true;
            //            vm.model.symbology = "SingleSymbol";

            //            SymbolService.createSymbol(newVal)
            //                .then(sym => {
            //                    vm.model.symbol = sym;
            //                });

            //            mzmap.refresh();

            //        }, reject => {

            //            vm.symbolType = oldVal;
            //        });
            //});

            //$scope.$watch(() => vm.model ? vm.model.symbology : null, (newVal, oldVal) => {

            //    var check = $util.isNullOrUndefined;

            //    if (vm._changeQueued ||
            //        check(newVal) ||
            //        (check(oldVal) && !check(newVal)) ||
            //        newVal == oldVal ||
            //        oldVal == "SingleSymbol") {
            //        vm._changeQueued = false;
            //        return;
            //    }

            //    dialog.confirm("Changing the symbology will clear all style values on this theme. Are you sure you want to proceeed?")

            //        .then(resolve => {
            //            vm.model.values = [];

            //        }, reject => {
            //            vm.model.symbology = oldVal;
            //        });

            //});